# Fssq (Famicom Sound Sequencer) 仕様書

## 概要

Fssqは、ファミコン/NES用の音楽ドライバです。
下記の機能をサポートしています。

* mapper
    * MMC3 (mapper4) に対応しています。
* BGM演奏
    * 曲データを元に
      2A03のpAPUの各音源(パルス波x2、三角波、ノイズ、DPCM)を
      制御して曲を演奏します。
* SE演奏
    * BGM演奏中にSE用の曲データを割りこませて演奏します。
    * 2A03のpAPUの各音源の内、DPCMを除いた音源を２つまで
      使用して演奏することが可能です。
* バンク切り替え
    * 演奏処理時、曲データを格納しているメモリバンクに
      自動でバンク切り替えを行い元に戻します。
* BGM演奏一時中断
    * BGM演奏を一時的に止めて再開することが可能です。
* BGM演奏フェードアウト
    * BGMの音量を徐々に小さくした後、演奏を停止することが可能です。

## 仕様

### 音長

本音楽ドライバでは、全音符の長さを192とするtickで
音長を管理します。

tickは整数で管理するため、割り切れない音長は利用できません。
(例 128b分音符=1.5tick など)

音長を保持するメモリ領域は8bitで全音符の長さの192tickまでを管理します。<br>
全音符を越える長さの音符/休符は、コンパイラにより音符/休符コマンドと
延長コマンドに分解されます。<br>
例えば、クォンタイズが7で480tick(全音符＋全音符＋二分音符)の音符は
```
音符コマンド(キーオン 96tick, キーオフ  0tick)
延長コマンド(キーオン192tick, キーオフ  0tick)
延長コマンド(キーオン168tick, キーオフ 24tick)
```
の３コマンドに分解されます。<br>
クォンタイズの指定は分割したコマンドの末尾のみ適用されます。

### チャンネル

本音楽ドライバでは、pAPUの各音源をチャンネルと呼びます。

|   |   |
|---|---|
|矩形波1|Aチャンネル|
|矩形波2|Bチャンネル|
|三角波 |Cチャンネル|
|ノイズ |Dチャンネル|

pAPUには上記の音源以外にDPCMが存在しますが、
本音楽ドライバでは独立したチャンネルとしては扱わず、
後述する特殊コマンドを使用して発音、制御します。

### トラック

本音楽ドライバでは、各チャンネル別のシーケンスデータの集まりを
トラックと呼びます。<br>
BGM演奏の場合には４トラック(内蔵音源のみの場合)まで使用できます。<br>
SE演奏の場合には２トラックまで使用できます。

### DPCM再生コールバック

本音楽ドライバでは、DPCMを発音する処理ルーチンは含めておらず、
ユーザが用意したコールバックルーチンをドライバに登録し、
音楽ドライバがDPCMを発音するタイミングでコールバックルーチンを
呼び出す方式を採用しています。

これは、本音楽ドライバを使用するプログラムの都合に合わせて
DPCMを再生させる方式を選択できるようにするためです。
例えば、$4012,$4013に値を書き込みファミコンのハードウェア機能を
利用した再生や、ソフトウェアで$4011に順じ値を設定して
PCMとして再生させる方式などがあります。

DPCM再生コールバックのインターフェイスは次の通りです。

|   |   |
|---|---|
|引数レジスタ  |Y : DPCM番号(0～127)|
|保持不要レジスタ  |A X Y|


## コンパイラ

mml2fssqは、テキストファイルに記述したMMLを
音楽ドライバ(sound.asm)が解釈可能なバイナリデータに変換する
コンパイラです。

このコンパイラは、nesasmでアセンブル可能なソースコードの
形式で次の２つのファイルを生成します。

* MLSDファイル
    * 曲データ本体のバイナリデータを定義するファイルです。
    * 曲データ本体は、下記の複数のデータテーブルから構成されます。
        * 音長テーブル
        * エンベロープテーブル
        * シーケンスデータ
* MLSCファイル
    * 下記の情報をまとめたテーブルを定義するファイルです。
        * 使用チャンネル情報
        * テンポ初期値
        * 曲データの各テーブルの先頭ラベル(アドレス)

MLSCファイルに定義されたテーブルの先頭アドレスを
Fssqの演奏開始サブルーチンに渡すことで、曲が演奏されます。


mml2fssqはコマンドプロンプトから実行します。
コマンド実行時のオプション指定は次の通りです。

```
mml2fssq <MMLファイル> <MLSCファイル> <MLSDファイル> <ラベル>
```

* <MMLファイル>
    * MMLを記述したテキストファイルのファイル名を指定します。
* <MLSCファイル>
    * MLSCファイルのファイル名を指定します。
* <MLSDファイル>
    * MLSDファイルのファイル名を指定します。
* <ラベル>
    * MLSCファイル、およびMLSDファイルに定義される各テーブルの
      先頭ラベルのプレフィクスを指定します。
      下記のラベルが生成されます。
        * <ラベル>_PlayData
        * <ラベル>_LenTable
        * <ラベル>_EnvTable
        * <ラベル>_SeqA, <ラベル>_SeqB, <ラベル>_SeqC, ...


## サブルーチン

音楽ドライバ(sound.asm)には、APIとして次のサブルーチンが
定義されています。

### Fssq_Init

音楽ドライバを初期化します。

|   |   |
|---|---|
|引数レジスタ  |なし|
|使用レジスタ  |A X|
|使用ゼロページ|なし|


### Fssq_SetDpcmCB

DPCM再生コールバックを設定します。

|   |   |
|---|---|
|引数レジスタ  |A : CBルーチンのアドレス(LOW)<br>X : CBルーチンのアドレス(HIGH)|
|使用レジスタ  |A X|
|使用ゼロページ|なし|


### Fssq_SetBankBGM

BGM用曲データのメモリバンクの番号と切り替えるバンク枠を指定します。

|   |   |
|---|---|
|引数レジスタ  |A : bbnnnnnn <br>b: 切り替えバンク枠の指定 (0:切り替えなし 1:$8000 2:$a000)<br>n: バンク番号 (0～63)|
|使用レジスタ  |A|
|使用ゼロページ|$00|

### Fssq_SetBankSE

SE用曲データのメモリバンクの番号と切り替えるバンク枠を指定します。

|   |   |
|---|---|
|引数レジスタ  |A : bbnnnnnn <br>b: 切り替えバンク枠の指定 (0:切り替えなし 1:$8000 2:$a000)<br>n: バンク番号 (0～63)|
|使用レジスタ  |A|
|使用ゼロページ|$00|

### Fssq_PlayBGM

BGMを演奏します。
曲データで定義された～_PlayDataのラベルのアドレスを引数で指定します。

|   |   |
|---|---|
|引数レジスタ  |A : 曲データのアドレス(LOW)<br>X : 曲データのアドレス(HIGH)|
|使用レジスタ  |A X Y|
|使用ゼロページ|$00 ～ $03 $0c ～$13|

### Fssq_PlaySE

SEを演奏します。
曲データで定義された～_PlayDataのラベルのアドレスを引数で指定します。

|   |   |
|---|---|
|引数レジスタ  |A : 曲データのアドレス(LOW)<br>X : 曲データのアドレス(HIGH)|
|使用レジスタ  |A X Y|
|使用ゼロページ|$00 ～ $03 $0c ～$13|

### Fssq_StopBGM

BGMの演奏を停止します。

|   |   |
|---|---|
|引数レジスタ  |なし|
|使用レジスタ  |A|
|使用ゼロページ|なし|

### Fssq_StopSE

SEの演奏を停止します。

|   |   |
|---|---|
|引数レジスタ  |なし|
|使用レジスタ  |A X Y|
|使用ゼロページ|なし|

### Fssq_PauseBGM

BGMの演奏を一時停止します。

|   |   |
|---|---|
|引数レジスタ  |なし|
|使用レジスタ  |A|
|使用ゼロページ|なし|

### Fssq_ResumeBGM

一時停止したBGMの演奏を再開します。

|   |   |
|---|---|
|引数レジスタ  |なし|
|使用レジスタ  |A|
|使用ゼロページ|なし|

### Fssq_Fadeout

演奏しているBGMの音量を徐々に小さくした後、BGMの演奏を停止します。

|   |   |
|---|---|
|引数レジスタ  |なし|
|使用レジスタ  |A|
|使用ゼロページ|なし|

### Fssq_Update

音楽ドライバの演奏処理を行います。
1/60秒毎にこのサブルーチンを呼び出してください。
音源のレジスタへの書き込みは本サブルーチンでは行わないため、
呼び出しタイミングは揺らぎがあっても問題ありません。
本サブルーチンの処理は割り込みがあっても問題ないため、
割り込み許可状態で呼びだすことを推奨します。

|   |   |
|---|---|
|引数レジスタ  |なし|
|使用レジスタ  |A X Y|
|使用ゼロページ|$00 ～ $0f|

### Fssq_Reflect

音楽ドライバが処理したデータを音源のレジスタに書き込みます。
1/60秒毎にこのサブルーチンを呼び出してください。
呼び出しタイミングはなるべく揺らぎがない方が正確な演奏となります。
NMI割り込みの延長で呼びだすことを推奨します。
本サブルーチンの処理は割り込みがあっても問題ないため、
割り込み許可状態で呼びだすことを推奨します。

|   |   |
|---|---|
|引数レジスタ  |なし|
|使用レジスタ  |A X Y|
|使用ゼロページ|なし|

## MML

コンパイラがサポートしているMML表記の詳細を以下に示します。

### データ定義

#### 書式

基本的な書式は次の通りです。

```
[ <定義名> [ <定義データ>... ] ] [ ; <コメント> ]
```

#### 定義名

記述可能な定義を以下に示します。

|定義名|説明|
|---|---|
|#EP|エンベロープデータ定義<br>発音中の音の音量や音程を変化させるエンベロープデータを定義します。|
|#RP|リズム定義<br>!コマンドで使用するリズムを定義します。

#### エンベロープデータ

エンベロープデータは、発音中の音量や音程の変化を付けるデータ定義です。<br>
時間の経過と共に音量や音程が変化することにより、音の聞こえ方、音色が変わります。

エンベロープデータ定義の書式は次の通りです。

```
#EP <番号> <減衰率> [ <変化値>... ] [ : <変化値>... ]
```

* <番号>
    * エンベロープデータを識別する番号で1～15の値を指定します。
* <減衰率>
    * 8bitの値で上位4bitと下位4bitのそれぞれに意味を持ちます。
    * 上位4bitはキーオン中の音量の減衰率を定義します。<br>ADSRのDとSを兼用したパラメータに相当します。<br>0で減衰なし。値が大きくなるほど減衰率は大きくなります。
    * 下位4bitはキーオフ中の音量の減衰率を定義します。<br>ADSEのRのパラメータに相当します。<br>0ですぐに音量0。値が大きくなるほど減衰率が小さくなります。
* <変化値>
    * 変化値は、音量係数、音色変更値、周波数増減値の３つのパラメータで構成されます。
    * 変化値は複数個並べて定義します。<br>発音すると1フレーム単位で複数並んだ変化値を順に参照し、変化値の定義に沿って音量や音程を変化させます。
    * 変化値の末尾まで参照が進むと、変化値の定義の間に : の記述の有無により動作が変わります。
        * : ありの場合は、: の次の変化値まで参照位置が戻ります。
        * : なしの場合は、変化なしの値を参照しているものして動作します。
    * 変化値は、音量係数、音色変更値、周波数増減値の順に / で区切って記述します。(例「4/12/1」)<br>周波数増減値が0の場合、２つ目の/から後の記述を省略出来ます。(例「4/12」)<br>音色変更値、周波数増減値の値が共に0の場合、１つ目の/から後の記述を省略出来ます。(例「4」)
    * 音量係数は、音量の増減率を指定します。<br>-4～8の値が指定可能で0は音量の変化は無く、0より小さい値は音量が小さくなり、0より大きい値は音量が大きくなります。<br>変化率は後述の表を参照ください。<br>内蔵音源の三角波は音量を制御できないため、音量係数は適用されません。
    * 音色変更値は4bitの値で、音源の種別により適用方法が異なります。
        * 内蔵音源の矩形波の場合は、$4000,$4004に設定する値の上位4bitに音色変更値をxorします。<br>$4000,$4004に設定する値の上位2bitがDuty比の設定であるため、矩形波の音色が変化します。
        * 内蔵音源の三角波とノイズは、適用されません。
    * 周波数増減値は、周波数の設定値を増減します。<br>-1024～1023の値が指定可能です。
        * 内蔵音源の矩形波の場合は、$4002,$4003,$4006,$4007に設定する値に周波数増減値を加算します。<br>レジスタに設定する値はシステムクロックを発音する音程の周波数で除算したものであるため、<br>値を増加した場合は低音に移行し、減算した場合は高音に移行します。
        * 内蔵音源の三角波の場合は、$400a,$400bに設定する値に周波数増減値を加算します。<br>レジスタに設定する値はシステムクロックを発音する音程の周波数で除算したものであるため、<br>値を増加した場合は低音に移行し、減算した場合は高音に移行します。
        * 内蔵音源のノイズの場合は、$400e,$400fに設定する値に周波数増減値を加算します。<br>レジスタに設定する値はシステムクロックを発音する音程の周波数で除算したものであるため、<br>値を増加した場合は低音に移行し、減算した場合は高音に移行します。

音量係数の増減率は次の通りです。

|音量係数|増減率|
|---|---|
| -4 | 0.5  倍 |
| -3 | 0.625倍 |
| -2 | 0.75 倍 |
| -1 | 0.875倍 |
|  0 | 1.0  倍 |
|  1 | 1.125倍 |
|  2 | 1.25 倍 |
|  3 | 1.375倍 |
|  4 | 1.5  倍 |
|  5 | 1.625倍 |
|  6 | 1.75 倍 |
|  7 | 1.875倍 |
|  8 | 2.0  倍 |


エンベロープデータの定義例を次に示します。

```
#EP 1 $c2
#EP 2 $42 4 1 0 3 1 : 0
#EP 3 $12 -4/8/1 -1/12/1 : 0/0/1
```

#### リズム

リズムデータは、Dチャンネルのシーケンスで、
定型のノイズやDPCMの発音を簡素に記述出来るよう
ノイズの音程、音量、エンベロープ指定、DPCMの打音指定の
組み合わせを定義するデータです。

リズム定義の書式は次の通りです。

```
#RP <名前> <打音番号> <音程> <音量> <エンベロープ指定> [ <クォンタイズ> ]
```

* <名前>
    * リズム定義データの名前を指定します。<br>アルファベット(A～Z a～z)とアンダースコア(_)の文字が使用できます。<br>大文字と小文字の区別は行いません。
* <打音番号>
    * DPCMで発音する音の番号を指定します。0～7の番号が指定できます。<br>指定した値-1が音の番号となります。0を指定するとDPCMの発音は行いません。
* <音程>
    * ノイズの音程を指定します。$00～$0f、$80～$8fの値を指定できます。
* <音量>
    * ノイズの音量を指定します。0～15の値を指定できます。
* <エンベロープ指定>
    * エンベロープデータの番号を指定します。<br>コンパイラが番号からエンベロープデータテーブルのインデックス値に自動変換しますが、リズム定義内の格納領域が小さいため、<br>エンベロープデータの１番目にリズム用のエンベロープデータを定義して、これを参照することを推奨します。
* <クォンタイズ>
    * ノイズのキーオン/キーオフの時間比を設定します。1～8の値が指定可能です。<br>値の詳細は後述のシーケンスのコマンドのクォンタイズの説明を参照下さい。<br>記述を省略した場合はMMLコマンド記述中のクォンタイズの指定値が適用されます。

リズムの定義例を次に示します。

```
;   name dpcm tone vol env quant
#RP b       1  $00   0   0       ; kick
#RP h       0  $01   6   1       ;       hi-hat
#RP bh      1  $01   6   1       ; kick  hi-hat
#RP hc      0  $01   4   1     3 ;       hi-hat close
#RP bhc     1  $01   4   1     3 ; kick  hi-hat close
#RP s       0  $07   4   1       ;       snare
#RP bs      1  $07   4   1       ; kick  snare
```

### シーケンス定義

#### 書式

基本的な書式は次の通りです。

```
[ <チャンネル> <MMLコマンド>... ] [ ; <コメント> ]
```

<チャンネル> にMMLの記述対象となるチャンネルを
アルファベットで指定します。

|   |   |
|---|---|
|A|矩形波1|
|B|矩形波2|
|C|三角波|
|D|ノイズ|

<MMLコマンド> にMMLコマンドを列記します。<br>
音楽ドライバは記述した順にMMLコマンドを実行し、音源を制御します。

; 以降はコメントとして無視されます。
コメントのみの行や空行の記述も可能です。

#### コマンド

記述可能なMMLコマンドを以下に示します。

|   |   |
|---|---|
|cdefgab|音符<br>コマンドの直後に+を付けると半音上がり-を付けると半音下がります。<br>コマンドの後に音長を指定することができます。<br>音長を指定しなかった場合にはlコマンドで指定したデフォルトの音長が適用されます。|
|r|休符<br>コマンドの後に音長を指定することができます。<br>音長を指定しなかった場合にはlコマンドで指定したデフォルトの音長が適用されます。|
|p|打音<br>DPCMを発音します。<br>コマンドの後にDPCM番号を指定することができます。<br>DPCM番号を指定しなかった場合は、DPCM番号0が適用されます。|
|!|リズム<br>リズム定義に従ってノイズとDPCMを発音します。<br>コマンド直後にリズム定義(#RP)の最初のパラメータのリズム名を記述します。<br>リズム名の後に音長を指定することができます。<br>Dチャンネルにのみ記述が可能です。
|&|タイ<br>先に発音した音符の発音時間を延長します。<br>コマンドの後に音長を指定することができます。<br>音長を指定しなかった場合にはlコマンドで指定したデフォルトの音長が適用されます。|
|m|参照演奏<br>本コマンドの直後に記述した番号と一致する()コマンドで範囲指定したデータを演奏します。<br>BGMでのみ使用できます。|
|o|オクターブ指定<br>コマンドの直後に記述した番号のオクターブを指定します。<br>番号は2～7の値が指定できます。|
|>|オクターブ上昇<br>現在のオクターブ指定を１つアップします。|
|<|オクターブ下降<br>現在のオクターブ指定を１つダウンします。|
|l|デフォルト音長の設定<br>音符/休符で音長を指定しなかった時に適用する音長を設定します。<br>コマンドの直後に音長を記述します。|
|q|クォンタイズ<br>キーオン/キーオフの時間比を設定します。<br>コマンドの直後に 1～8 の値を指定します。<br>指定した値をnとした場合、キーオンの時間(GateTime)は音長(StepTime)のn/8、<br>キーオフの時間(StepTime-GateTime)は音長の(8-n)/8となります。<br>8を指定した場合、音長の全時間がキーオンの状態となります。|
|t|テンポ<br>テンポを指定します。<br>コマンドの直後に 1～255 の値を指定します。<br>Aチャンネルに指定してください。|
|@x|レジスタ1設定<br>内蔵音源は$4000,$4004,$4008,$400cのレジスタに値を設定します。<br>コマンド直後にレジスタに設定する値を指定します。<br><br>矩形波およびノイズではbit5(キーオフカウンタフラグ)、bit4(エンベロープループカウンタ)に1を設定して機能を無効にしてください。<br>三角波では$ffを設定してください。|
|@y|レジスタ2設定<br>内蔵音源は$4001,$4005のレジスタに値を設定します。<br>コマンド直後にレジスタに設定する値を指定します。<br><br>矩形波では、ハードウェアのスイープ機能を使用せず広域リピート機能を使用する場合は$00を設定してください。<br>ハードウェアのスイープ機能を使用せずo2g+以下の低音を使用する場合は$08を設定します。<br>三角波およびノイズではハードウェア的に未使用レジスタであるため、設定しても無意味です。<br>値の保存領域は広域リピート機能のカウンタとして利用するので本コマンドは使用しないでください。|
|@v+|音量増加1<br>音源の音量を１増加します。|
|@v++|音量増加2<br>音源の音量を２増加します。|
|@v-|音量減少1<br>音源の音量を１減少します。|
|@v--|音量減少2<br>音源の音量を２減少します。|
|@ev|エンベロープ設定<br>適用するエンベロープデータを設定します。<br>コマンド直後にエンベロープデータの番号を指定します。<br>番号に0を指定すると適用なしとなります。|
|@lc|ループカウンタ更新<br>曲のループ回数を手動更新します。<br>仕様上チャンネル毎にループタイミングを変えることが可能で、どのタイミングを曲のループとするか定義できず、<br>自動検出が困難なため、ユーザがタイミングを指示する方式としています。|
|[ \| ]|リピート<br>[ と ] の間に挟まれたMMLを繰り返し演奏します。<br>]の直後に繰り返し回数を指定します。<br>[ と ] の間に \| が記述されていると、最後の繰り返しでは、\| と ] の間のMMLをスキップします。|
|@[ @\| @]|広域リピート<br>@[ と @] の間に挟まれたMMLを繰り返し演奏します。<br>@]の直後に繰り返し回数を指定します。<br>@[ と @] の間に @\| が記述されていると、最後の繰り返しでは、@\| と @] の間のMMLをスキップします。<br><br>カウンタの保存領域を@yコマンドで設定するレジスタ値と共有しているため、<br>内蔵音源の矩形波で使用する場合には次の制限があります。<br>・ハードウェアのスイープ機能を使用する場合には広域リピート機能を使用できません。<br>・o2g+以下の低音を発音する場合には広域リピート機能を使用できません。<br>・広域リピート機能を使用する場合には@yコマンドで$00を設定し、カウンタを初期化します。|
|( )|参照範囲指定<br>mコマンドで参照するコマンドの範囲を指定します。<br>)の直後に識別の番号を指定します。|
|:|トラックループ<br>このコマンドが記述されているとデータ終端まで演奏した後、このコマンドの位置に戻ります。<br>このコマンドが記述されていない場合は、データ終端まで演奏した後、演奏を停止します。|
|/|データ終端<br>通常は各トラックのデータの入力末尾がデータ終端として自動的に認識しますが、<br>参照範囲指定するデータをトラックの末尾に集約して、<br>集約したデータの手前でトラックループまたは演奏停止を行いたい場合に<br>このコマンドを記述します。|


### MMLのサンプル

MMLの記述例を以下に示します。

```
; 3人の王の行列

#EP 1 $c2
#EP 2 $42 4 2 0 3 1 : 0
#EP 3 $a2 4/8 2/12 0 3 1 : 0

A t150 @x$b6 @y$08

A o4 q7 @ev3
A :
A ( r16 f16 f4 c4 
A f4.. g16 g+8. g16 g+8. f16 
A > c4 < r8 g+8 a+4 > c4 <
A > c+8 c8 < a+8 g+8 )1 g4 a+4
A a+8 g+8 g8
A m1
A g+4 g4 f4.

A ( r16 g16 g4 g4 
A g+4 g8. f16 g4 g+4
A a+4. r16 g+16 a+4 > c4 < )2
A f4 g8 g+8 a+8 g+8 g8 f8
A f8 e8 c8
A m2
A > c+8 c8 < a+8 g+8 g+4 g4 f4.

```


## バイナリデータ

本音楽ドライバ(sound.asm)が解釈可能なバイナリデータの仕様を以下に説明します。

### 構成

バイナリデータは、次の３種類のデータで構成されます。

* 音長テーブル
* エンベロープテーブル
* シーケンスデータ

音長テーブル、および、エンベロープテーブルは
曲全体で１つずつ存在します。<br>
シーケンスデータは、使用するチャンネル分存在します。

### 音長テーブル

音長テーブルは、演奏する各音の演奏時間を収集して
テーブル化したものです。<br>
また、リズムを定義した場合に定義内容をバイナリ化したものも、
このテーブルに格納されます。

音長データは、キーオンとキーオフの各tickを8bit値のペアで定義します。<br>
四分音符のtickは48ですが、クォンタイズが7とした場合、
キーオンは42tick、キーオフは6tickになるため、
音長テーブルには次のように定義されます。

```
	.db	42, 6
```

リズムは１定義辺り16bit値で定義されます。<br>
16bit値のフォーマットは次の通りです。

```
15      8 7      0
 tppptttt vvvveeee

p　　   14..12bit : 打音-1(0..7)
t　　15,11.. 8bit : ノイズの音程($00..$0f,$80..$8f)
v　　    7.. 4bit : ノイズの音量(0..15)
e　　    3.. 0bit : エンベロープベース値(0..15)
```

エンベロープベース値は、エンベロープテーブル内のエンベロープデータ先頭を指す位置です。

### エンベロープテーブル

エンベロープテーブルは、エンベロープデータの定義をバイナリの変換し、
まとめたテーブルです。<br>
エンベロープデータは8bit値の任意長のシーケンスデータに変換されます。
本音楽ドライバは、このシーケンスデータを順に読み込み解釈することで
発音した音の音程や音量を制御します。<br>
エンベロープデータのシーケンスを構成する8bit値の種類は次の通りです。

|形式|条件|説明|
|---|---|---|
|xxxxyyyy|シーケンスデータの先頭1byte目|x : キーオン中の音量の減衰率<br>y : キーオフ中の音量の減衰率|
|vvvv0000|シーケンスデータの2byte目以降<br>vが$0～$c|v : 変化値の音量係数(4～15)<br>変化値の音色変更値、周波数増減値は0|
|vvvv0010 ccccdddd|シーケンスデータの2byte目以降<br>vが$0～$c|v : 変化値の音量係数(4～15)<br>c : 変化値の音色変更値(0～15)<br>d : 変化値の周波数増減値(-8～7)|
|vvvv0001 ccccdddd dddddddd|シーケンスデータの2byte目以降<br>vが$0～$c|v : 変化値の音量係数(4～15)<br>c : 変化値の音色変更値(0～15)<br>d : 変化値の周波数増減値(-2048～2047)|
|xxxxxxxx|シーケンスデータの2byte目以降<br>xが$d0～$fe|シーケンスデータの位置移動<br>xの値(-48～-2)を加算して位置を巻き戻します。|
|11111111|シーケンスデータの2byte目以降|変化値の音量係数、音色変更値、周波数増減値を0とし、<br>シーケンスの位置を移動しません。|


### シーケンスデータ

シーケンスデータは、各チャンネルの発音を制御するコマンドが並んだデータ列です。<br>
本音楽ドライバは、このシーケンスデータを順に読み込み解釈することで
曲の演奏を行います。

シーケンスデータのコマンドは次の４種類に大きく別れます。
* 音符コマンド
* 休符コマンド
* 延長コマンド
* 特殊コマンド

上記コマンドの定義域は次の通りです。

```
     +0 +1 +2 +3 +4 +5 +6 +7 +8 +9 +A +B +C +D +E +F
    +-----------------------------------------------+
$00 |                  特殊コマンド                 |
$10 |                                               |
    +--+--+-----------------------------------------+
$20 |  |  |                                         |
$30 |  |  |                                         |
$40 |  |  |                                         |
$50 |  |  |                                         |
$60 |延|休|                                         |
$70 |長|符|                                         |
$80 |コ|コ|            音符コマンド                 |
$90 |マ|マ|                                         |
$A0 |ン|ン|                                         |
$B0 |ド|ド|                                         |
$C0 |  |  |                                         |
$D0 |  |  |                                         |
$E0 |  |  |                                         |
$F0 |  |  |                                         |
    +--+--+-----------------------------------------+
```

#### 音符コマンド

発音を指示するコマンドです。

コマンドは8bit長でフォーマットは次の通りです。

```
xxxxyyyy

x : 音長差分値
y : 音程差分値
```

コマンドの上位4bitの音長差分値(2～15)から2を引いて
音長ベース値を加算した値をインデックスとして、
音長テーブルからキーオン/キーオフのtickを表引きします。<br>
表引きしたtickの分の時間、発音します。

コマンドの下位4bitの音程差分値(2～15)から2を引いて
音程ベース値を加算した値を音程として発音します。<br>
音程は0～71の値でo2c～o7bの音程に対応します。

矩形波、および三角波では、音程をインデックスとして
音楽ドライバ内の周波数テーブルを表引きして周波数を求め、
レジスタに書き込みます。

ノイズは、音程が31以下の場合にはノイズ周波数指定、
32以上の場合にはリズム指定となります。

ノイズ周波数指定は、音程($00～$1f)から$10を引いて
$8fとANDをとった値($80～$8f,$00～$0f)をレジスタに書き込みます。<br>
したがって、音程(0～15 o2c～o3d+)が短周期ノイズ、
音程(16～31 o3e～o4g)が長周期ノイズとなります。

リズム指定は、音程($20～)から$20を引いた値をインデックスとして
音長テーブルに定義されたリズム定義を表引きし、
リズム定義の情報に従って、ノイズの音程や音量を設定します。


#### 休符コマンド

無音を指示するコマンドです。

コマンドは8bit長でフォーマットは次の通りです。

```
xxxx0001

x : 音長差分値
```

コマンドの上位4bitの音長差分値(2～15)から2を引いて
音長ベース値を加算した値をインデックスとして、
音長テーブルからキーオン/キーオフのtickを表引きします。<br>
表引きしたtickの分の時間、音を止めます。

#### 延長コマンド

発音または無音の時間を延長するコマンドです。

コマンドは8bit長でフォーマットは次の通りです。

```
xxxx0000

x : 音長差分値
```

コマンドの上位4bitの音長差分値(2～15)から2を引いて
音長ベース値を加算した値をインデックスとして、
音長テーブルからキーオン/キーオフのtickを表引きします。<br>
表引きしたtickをカウンタに加算し、発音または無音中の時間を延長します。

#### 特殊コマンド

発音や無音以外のシーケンス制御に必要なコマンドをまとめたグループです。

特殊コマンドの詳細を下記に示します。

|コマンド名|フォーマット|説明|
|---|---|---|
| 終端コマンド | $00 | シーケンスの終端を示すコマンドです。<br>チャンネルの演奏を停止します。|
| リターンコマンド | $01 | サブルーチン呼び出し中の場合は呼び出し元に戻ります。<br> サブルーチン呼び出し中でない場合は何も処理しません。 |
| サブルーチンコマンド | $02 $nn [ $nn ] | サブルーチン制御を行います。<br>現在のシーケンス位置を保存しジャンプします。<br>2バイトサイズのショート形式と3バイトサイズのロング形式の２種類があります。<br><br>ショート形式 (-64～63)<br>2バイト目7bit : 0<br>2バイト目6～0bit : ジャンプ先相対位置<br><br>ロング形式 (-16384～16383)<br>2バイト目7bit : 1<br>2バイト目6～0 bit : ジャンプ先相対位置 下位7bit<br>3バイト目7～0 bit : ジャンプ先相対位置 上位8bit |
| ジャンプコマンド | $03 $nn [ $nn ] | ジャンプ制御を行います。<br>2バイトサイズのショート形式と3バイトサイズのロング形式の２種類があります。<br>フォーマットはサブルーチンコマンドと同じです。 |
| リピートコマンド(-2,3回)| $04 | 特定パターン専用のリピート制御です。戻り位置-2バイトで繰り返し回数3回です。|
| リピートコマンド(-2,4回)| $05 | 特定パターン専用のリピート制御です。戻り位置-2バイトで繰り返し回数4回です。|
| リピートコマンド(-3,2回)| $06 | 特定パターン専用のリピート制御です。戻り位置-3バイトで繰り返し回数2回です。|
| リピートコマンド(-3,3回)| $07 | 特定パターン専用のリピート制御です。戻り位置-3バイトで繰り返し回数3回です。|
| リピートコマンド(-4,2回)| $08 | 特定パターン専用のリピート制御です。戻り位置-4バイトで繰り返し回数2回です。|
| リピートコマンド(-4,3回)| $09 | 特定パターン専用のリピート制御です。戻り位置-4バイトで繰り返し回数3回です。|
| リピートコマンド<br>(ショート形式) | $0a $nn | リピート制御を行います。<br><br>2バイト目 7～5bit : 繰り返し回数(2～9回)<br>2バイト目 4～0bit : 戻り位置(-32～-1バイト)|
| リピートコマンド<br>(ロング形式) | $0b $nn $nn | リピート制御を行います。<br><br>2バイト目 7～0bit : 繰り返し回数(2～127回)<br>3バイト目 7～0bit : 戻り位置(-256～-1バイト)|
| 広域リピートコマンド<br>(ショート形式) | $0c $nn | リピート制御を行います。<br><br>2バイト目 7～5bit : 繰り返し回数(2～9回)<br>2バイト目 4～0bit : 戻り位置(-32～-1バイト)|
| 広域リピートコマンド<br>(ロング形式) | $0d $nn $nn | リピート制御を行います。<br><br>2バイト目 7～0bit : 繰り返し回数(2～127回)<br>3バイト目 7～0bit : 戻り位置(-256～-1バイト)|
| レジスタ1設定コマンド | $10 $nn | $4000(矩形波1)、$4004(矩形波2)、$4008(三角波)、$400c(ノイズ)のレジスタに値($nn)を設定します。 |
| レジスタ2設定コマンド | $11 $nn | $4001(矩形波1)、$4005(矩形波2)、$4009(三角波)、$400d(ノイズ)のレジスタに値($nn)を設定します。<br>値の保存領域を広域リピート機能のカウンタと共有しているため、カウンタの初期化としても使用できます。 |
| 音程ベース設定コマンド | $12 $nn | 音程ベース値($nn)を設定するコマンドです。 |
| 音長ベース設定コマンド | $13 $nn | 音長ベース値($nn)を設定するコマンドです。 |
| エンベロープベース設定コマンド | $14 $nn | エンベロープベース値($nn)を設定するコマンドです。 |
| テンポ設定コマンド | $15 $nn | テンポ($nn)を設定するコマンドです。 |
| 打音コマンド | $16 $nn | DPCMで発音するコマンドです。<br>DPCM再生コールバックに渡す値($nn)を保持します。 |
| ループカウンタコマンド | $17 | 曲のループ回数のカウンタを1増加します。|
| 音量変更コマンド(+1) | $18 | 音量を1増加します。 |
| 音量変更コマンド(-1) | $19 | 音量を1減少します。 |
| 音量変更コマンド(+2) | $1a | 音量を2増加します。 |
| 音量変更コマンド(-2) | $1b | 音量を2減少します。 |

